#include "rgr_cfg.h"
#include "rgr_led.h"
#include "rgr_cmd.h"

#include <stdio.h>
#include <string.h>
#include "rgr_error.h"
#include "nrf_error.h"
#include "pstorage.h"

static pstorage_handle_t m_storage_handle;

// pattern for inizialized configuration
static const uint8_t m_config_pattern[] = { 0x5A, 0x5A };

// default configuration for leds and motor (full intensity)
/*static const uint8_t m_config_default[] __attribute__((aligned(4))) = 
	{ 
		0x5A, 0x5A, 		// pattern
		0xFF, 0xFF, 0xFF, 	// leds_indication_intensity
		0xFF, 0xFF, 0xFF, 	// leds_command_intensity
		0xFF 				// motor_intensity
	};
*/

static const rgr_cfg_config_t m_config_default __attribute__((aligned(4))) = 
	{ 
		.pattern = { 0x5A, 0x5A }, 							// pattern
		.leds_indication_intensity = { 0xFF, 0xFF, 0xFF }, 	// leds_indication_intensity
		.leds_command_intensity = { 0xFF, 0xFF, 0xFF }, 	// leds_command_intensity
		.motor_intensity = 0xFF 							// motor_intensity
	};

// configuration in memory cache
static rgr_cfg_config_t m_config __attribute__((aligned(4)));

static void rgr_cfg_pstorage_cb_handler(pstorage_handle_t *p_handle,
										uint8_t op_code,
										uint32_t result,
										uint8_t *p_data,
										uint32_t data_len)
{
	printf("p_handle=%d, op_code=%d, result=%d, p_data=%d, data_len=%d\r\n", p_handle, op_code, result, p_data, data_len);
	
	for (int i = 0; i < data_len; i++)
	{
		printf("%d\r\n", p_data[i]);
		//printf("p_data[%d] = %d \r\n", i, p_data[i]);
	}
}

void rgr_cfg_save()
{
	uint32_t err_code = NRF_SUCCESS;
	pstorage_handle_t block_handle;
	
	err_code = pstorage_block_identifier_get(&m_storage_handle, 0, &block_handle);
	printf("pstorage_block_identifier_get err_code = %d\r\n", err_code);
	printf("block_handle.block_id = %d\r\n", block_handle.block_id);
	
	uint8_t data[] = { 0x10, 0x20, 0x30, 0x40 };
	
	err_code = pstorage_store(&block_handle, data, 4, 0);
	printf("pstorage_store err_code = %d\r\n", err_code);
	printf("block_handle.block_id = %d\r\n", block_handle.block_id);
}

void rgr_cfg_load()
{
	uint32_t err_code = NRF_SUCCESS;
	pstorage_handle_t block_handle;
	uint8_t data[4];
	
	err_code = pstorage_block_identifier_get(&m_storage_handle, 0, &block_handle);
	printf("pstorage_block_identifier_get err_code = %d\r\n", err_code);
	printf("block_handle.block_id = %d\r\n", block_handle.block_id);
	
	err_code = pstorage_load(data, &block_handle, 4, 0);
	printf("pstorage_load err_code = %d\r\n", err_code);
	
	/*
	for (int i = 0; i < 4; i++)
	{
		printf("p_data[%i] = %x\r\n", i, data[i]);
	}
	*/
}

uint32_t rgr_cfg_init()
{
	uint32_t err_code = RGR_SUCCESS;
	uint8_t pattern[RGR_CONFIG_PATTERN_SIZE];
	pstorage_handle_t block_handle;
	pstorage_module_param_t param;
	
	volatile uint8_t *ptr = (uint8_t *)260096;
	
	//memcpy((uint8_t *)&m_config, (uint8_t *)&m_config_default, RGR_CONFIG_SIZE);
	
	// TODO : load configuration from flash
	
	//rgr_led_config(m_config.leds_indication_intensity, RGR_CONFIG_INDICATION_LEDS_SIZE);
	//rgr_cmd_config(m_config.leds_command_intensity, RGR_CONFIG_COMMAND_LEDS_SIZE, m_config.motor_intensity);
	
	// pstorage for configuration (only 1 block of 16 bytes)
	param.block_size = PSTORAGE_MIN_BLOCK_SIZE;
	param.block_count = 1;
	param.cb = rgr_cfg_pstorage_cb_handler;
	
	err_code = pstorage_register(&param, &m_storage_handle);
	//printf("pstorage_register err_code = %d\r\n", err_code);
	//printf("m_storage_handle.block_id = %d\r\n", m_storage_handle.block_id);

	if(*ptr != 16) {
		printf("rgr_cfg_save\r\n");
		rgr_cfg_save();
	} else {
		printf("rgr_cfg_load\r\n");
		rgr_cfg_load();
	}
	
	/*
	if (err_code == NRF_SUCCESS)
	{	
		err_code = pstorage_block_identifier_get(&m_storage_handle, 0, &block_handle);
		printf("pstorage_block_identifier_get err_code = %d\r\n", err_code);
		printf("block_handle.block_id = %d\r\n", block_handle.block_id);
		
		err_code = pstorage_load(pattern, &block_handle, RGR_CONFIG_PATTERN_SIZE, RGR_CONFIG_PATTERN_OFFSET);
		printf("pstorage_load err_code = %d\r\n", err_code);
		printf("pattern[0] = %d, pattern[1] = %d\r\n", pattern[0], pattern[1]);
		
		// check if configuration is already initialized
		if (memcmp(pattern, m_config_pattern, sizeof(m_config_pattern)) != 0)
		{
			uint8_t config_default[] = { 0x10, 0x20, 0x30, 0x40 }; //, 0x50, 0x60, 0x70, 0x80, 0x90 };
			//err_code = pstorage_store(&block_handle, (uint8_t *)m_config_default, sizeof(m_config_default), RGR_CONFIG_START_OFFSET);
			err_code = pstorage_store(&block_handle, config_default, sizeof(config_default), 0);
			printf("block_handle.block_id err_code = %d\r\n", block_handle.block_id);
		}
		else
		{
			err_code = pstorage_load((uint8_t *)&m_config, &block_handle, sizeof(m_config), RGR_CONFIG_PATTERN_OFFSET);
			printf("pstorage_store err_code = %d\r\n", err_code);
		}
		
	}
	*/
	
	err_code = (err_code == NRF_SUCCESS) ? RGR_SUCCESS : RGR_ERROR_STORAGE;
	
	return err_code;
}

/*
uint32_t rgr_cfg_save(uint8_t *p_data, uint8_t length)
{
	uint32_t err_code = RGR_SUCCESS;
	
	if (p_data == NULL || length == 0)
		return RGR_ERROR_WRONG_PARAMETER;
	
	switch (p_data[RGR_CONFIG_CODE_OFFSET])
	{
		case RGR_CONFIG_INDICATION_LEDS_CODE:
			
			if (length - 1 != RGR_CONFIG_INDICATION_LEDS_SIZE)
				return RGR_ERROR_WRONG_PARAMETER;
			
			// intensity bytes start from the second position
			memcpy(m_config.leds_indication_intensity, p_data + 1, RGR_CONFIG_INDICATION_LEDS_SIZE);
			rgr_led_config(m_config.leds_indication_intensity, RGR_CONFIG_INDICATION_LEDS_SIZE);
			
			break;
		
		case RGR_CONFIG_COMMAND_LEDS_CODE:
		
			if (length - 1 != RGR_CONFIG_COMMAND_LEDS_SIZE)
				return RGR_ERROR_WRONG_PARAMETER;
			
			// intensity bytes start from the second position
			memcpy(m_config.leds_command_intensity, p_data + 1, RGR_CONFIG_COMMAND_LEDS_SIZE);
			rgr_cmd_config(m_config.leds_command_intensity, RGR_CONFIG_COMMAND_LEDS_SIZE, m_config.motor_intensity);
		
			break;
		
		case RGR_CONFIG_COMMAND_MOTOR_CODE:
		
			if (length - 1 != RGR_CONFIG_COMMAND_MOTOR_SIZE)
				return RGR_ERROR_WRONG_PARAMETER;
			
			m_config.motor_intensity = *p_data;
			rgr_cmd_config(m_config.leds_command_intensity, RGR_CONFIG_COMMAND_LEDS_SIZE, m_config.motor_intensity);
		
			break;
	}
	
	// TODO : save configuaration in flash
	
	return err_code;
}
*/
